import React from 'react';
import { useRerenderer } from './utils';
import { getResolvedQueryConfig } from '../core/config';
import { QueryObserver } from '../core/queryObserver';
import { useErrorResetBoundary } from './ReactQueryErrorResetBoundary';
import { useQueryCache } from '.';
import { useContextConfig } from './ReactQueryConfigProvider';
export function useBaseQuery(queryKey, config) {
  var cache = useQueryCache();
  var rerender = useRerenderer();
  var contextConfig = useContextConfig();
  var errorResetBoundary = useErrorResetBoundary(); // Get resolved config

  var resolvedConfig = getResolvedQueryConfig(cache, queryKey, contextConfig, config); // Create query observer

  var observerRef = React.useRef();
  var firstRender = !observerRef.current;
  var observer = observerRef.current || new QueryObserver(resolvedConfig);
  observerRef.current = observer; // Subscribe to the observer

  React.useEffect(function () {
    return observer.subscribe(function () {
      rerender();
    });
  }, [observer, rerender]); // Update config

  if (!firstRender) {
    observer.updateConfig(resolvedConfig);
  }

  var result = observer.getCurrentResult(); // Handle suspense

  if (resolvedConfig.suspense || resolvedConfig.useErrorBoundary) {
    var query = observer.getCurrentQuery();

    if (result.isError && !errorResetBoundary.isReset() && query.state.throwInErrorBoundary) {
      throw result.error;
    }

    if (resolvedConfig.enabled && resolvedConfig.suspense && !result.isSuccess) {
      errorResetBoundary.clearReset();
      var unsubscribe = observer.subscribe();
      throw observer.fetch().finally(unsubscribe);
    }
  }

  return result;
}