"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.useBaseQuery = useBaseQuery;

var _react = _interopRequireDefault(require("react"));

var _utils = require("./utils");

var _config = require("../core/config");

var _queryObserver = require("../core/queryObserver");

var _ReactQueryErrorResetBoundary = require("./ReactQueryErrorResetBoundary");

var _ = require(".");

var _ReactQueryConfigProvider = require("./ReactQueryConfigProvider");

function useBaseQuery(queryKey, config) {
  var cache = (0, _.useQueryCache)();
  var rerender = (0, _utils.useRerenderer)();
  var contextConfig = (0, _ReactQueryConfigProvider.useContextConfig)();
  var errorResetBoundary = (0, _ReactQueryErrorResetBoundary.useErrorResetBoundary)(); // Get resolved config

  var resolvedConfig = (0, _config.getResolvedQueryConfig)(cache, queryKey, contextConfig, config); // Create query observer

  var observerRef = _react.default.useRef();

  var firstRender = !observerRef.current;
  var observer = observerRef.current || new _queryObserver.QueryObserver(resolvedConfig);
  observerRef.current = observer; // Subscribe to the observer

  _react.default.useEffect(function () {
    return observer.subscribe(function () {
      rerender();
    });
  }, [observer, rerender]); // Update config


  if (!firstRender) {
    observer.updateConfig(resolvedConfig);
  }

  var result = observer.getCurrentResult(); // Handle suspense

  if (resolvedConfig.suspense || resolvedConfig.useErrorBoundary) {
    var query = observer.getCurrentQuery();

    if (result.isError && !errorResetBoundary.isReset() && query.state.throwInErrorBoundary) {
      throw result.error;
    }

    if (resolvedConfig.enabled && resolvedConfig.suspense && !result.isSuccess) {
      errorResetBoundary.clearReset();
      var unsubscribe = observer.subscribe();
      throw observer.fetch().finally(unsubscribe);
    }
  }

  return result;
}